@@ -70,10 +70,12 @@ static volatile int gli_window_alive = TRUE;
 - (void) closeWindow: (pid_t) processID;
 
 - (NSString *) openWindowDialog: (pid_t) processID
-                         prompt: (NSString *) prompt;
+                         prompt: (NSString *) prompt
+                         filter: (unsigned int) filter;
 
 - (NSString *) saveWindowDialog: (pid_t) processID
-                         prompt: (NSString *) prompt;
+                         prompt: (NSString *) prompt
+                         filter: (unsigned int) filter;
 
 - (void) abortWindowDialog: (pid_t) processID
                     prompt: (NSString *) prompt;
@@ -214,12 +216,13 @@ void winexit(void)
     exit(0);
 }
 
-void winopenfile(char *prompt, char *buf, int len, char *filter)
+void winopenfile(char *prompt, char *buf, int len, int filter)
 {
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];    
 
     NSString * fileref = [gargoyle openWindowDialog: processID
-                                             prompt: [NSString stringWithCString: prompt encoding: NSASCIIStringEncoding]];
+                                             prompt: [NSString stringWithCString: prompt encoding: NSASCIIStringEncoding]
+                                             filter: filter];
 
     strcpy(buf, "");
 
@@ -238,12 +241,13 @@ void winopenfile(char *prompt, char *buf, int len, char *filter)
     [pool drain];
 }
 
-void winsavefile(char *prompt, char *buf, int len, char *filter)
+void winsavefile(char *prompt, char *buf, int len, int filter)
 {
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
     NSString * fileref = [gargoyle saveWindowDialog: processID
-                                             prompt: [NSString stringWithCString: prompt encoding: NSASCIIStringEncoding]];
+                                             prompt: [NSString stringWithCString: prompt encoding: NSASCIIStringEncoding]
+                                             filter: filter];
 
     strcpy(buf, "");
 
@@ -482,7 +486,7 @@ void winkey(NSEvent *evt)
     gli_refresh_needed = TRUE;
 
     /* check for arrow keys */
-    if ([evt modifierFlags] & NSNumericPadKeyMask)
+    if ([evt modifierFlags] & NSFunctionKeyMask)
     {
         /* modified keys for scrolling */
         if ([evt modifierFlags] & NSCommandKeyMask
@@ -704,7 +708,7 @@ void winpoll(void)
     do
     {
         if (gli_refresh_needed)
-            winrefresh(); 
+            winrefresh();
 
         if (gli_event_waiting)
             evt = [gargoyle getWindowEvent: processID];
@@ -721,7 +725,10 @@ void gli_select(event_t *event, int polled)
     gli_curevent = event;
     gli_event_clearevent(event);
 
-    if (!polled)
+    winpoll();
+    gli_dispatch_event(gli_curevent, polled);
+
+    if (gli_curevent->type == evtype_None && !polled)
     {
         while (![monitor timeout])
         {
@@ -735,13 +742,6 @@ void gli_select(event_t *event, int polled)
         }
     }
 
-    else
-    {
-        if (![monitor timeout])
-            winpoll();
-        gli_dispatch_event(gli_curevent, polled);
-    }
-
     if (gli_curevent->type == evtype_None && [monitor timeout])
     {
         gli_event_store(evtype_Timer, NULL, 0, 0);
